"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const execa_1 = __importDefault(require("execa"));
const build_utils_1 = require("@now/build-utils");
exports.version = process.env.RUNTIME_NAME ? 3 : 1;
async function build(opts) {
    if (exports.version !== 3) {
        console.error('now-deno only supports v3 ZEIT Now builders');
        throw Error();
    }
    const lambdaFiles = await getDenoLambdaLayer(opts);
    const { files, entrypoint, workPath, config, meta = {} } = opts;
    console.log('downloading files');
    const downloadedFiles = await build_utils_1.download(files, path_1.default.join(workPath, 'src'), meta);
    const entryPath = downloadedFiles[entrypoint].fsPath;
    await runUserScripts(entryPath);
    const extraFiles = await gatherExtraFiles(config.includeFiles, entryPath);
    return buildDenoLambda(opts, downloadedFiles, extraFiles, lambdaFiles);
}
exports.build = build;
async function buildDenoLambda({ workPath, entrypoint, config }, downloadedFiles, extraFiles, layerFiles) {
    console.log('building single file');
    const entrypointPath = downloadedFiles[entrypoint].fsPath;
    const entrypointDirname = path_1.default.dirname(entrypointPath);
    const binName = path_1.default.basename(entrypointPath);
    const binPath = path_1.default.join(workPath, binName);
    const { debug } = config;
    console.log('running `deno bundle`...');
    try {
        await execa_1.default(path_1.default.join(workPath, 'layer', 'amz-deno'), ['bundle', entrypointPath, binPath].concat(debug ? ['-L debug'] : []), {
            env: {
                DENO_DIR: path_1.default.join(workPath, 'layer', '.deno_dir'),
            },
            cwd: entrypointDirname,
            stdio: 'inherit',
        });
    }
    catch (err) {
        console.error('failed to `deno build`');
        throw err;
    }
    const extname = path_1.default.extname(entrypointPath);
    const lambda = await build_utils_1.createLambda({
        files: {
            ...extraFiles,
            ...layerFiles,
            [binName]: new build_utils_1.FileFsRef({
                mode: 0o755,
                fsPath: binPath,
            }),
        },
        handler: binName.replace(extname, '.handler'),
        runtime: 'provided',
    });
    console.log(lambda);
    if (exports.version === 3) {
        return { output: lambda };
    }
    return {
        [entrypoint]: lambda,
    };
}
async function getDenoLambdaLayer({ workPath }) {
    const zipPath = path_1.default.join(workPath, 'deno-lambda-layer.zip');
    try {
        await execa_1.default('curl', [
            '-o',
            zipPath,
            '-L',
            'https://github.com/hayd/deno-lambda/releases/latest/download/deno-lambda-layer.zip',
        ], {
            stdio: 'inherit',
        });
    }
    catch (err) {
        console.error('failed to download `deno-lambda-layer.zip`');
        throw err;
    }
    const layerDir = path_1.default.join(workPath, 'layer');
    try {
        await execa_1.default('unzip', [zipPath, '-d', layerDir], {
            stdio: 'inherit',
        });
    }
    catch (err) {
        console.error('failed to unzip `deno-lambda-layer.zip` into `layer`');
        throw err;
    }
    return {
        bootstrap: new build_utils_1.FileFsRef({
            mode: 0o755,
            fsPath: path_1.default.join(layerDir, 'bootstrap'),
        }),
        'amz-deno': new build_utils_1.FileFsRef({
            mode: 0o755,
            fsPath: path_1.default.join(layerDir, 'amz-deno'),
        }),
    };
}
async function runUserScripts(entrypoint) {
    const entryDir = path_1.default.dirname(entrypoint);
    const buildScriptPath = path_1.default.join(entryDir, 'build.sh');
    const buildScriptExists = await fs_extra_1.default.pathExists(buildScriptPath);
    if (buildScriptExists) {
        console.log('running `build.sh`...');
        await build_utils_1.runShellScript(buildScriptPath);
    }
}
async function gatherExtraFiles(globMatcher, entrypoint) {
    if (!globMatcher)
        return {};
    console.log('gathering extra files for the fs...');
    const entryDir = path_1.default.dirname(entrypoint);
    if (Array.isArray(globMatcher)) {
        const allMatches = await Promise.all(globMatcher.map(pattern => build_utils_1.glob(pattern, entryDir)));
        return allMatches.reduce((acc, matches) => ({ ...acc, ...matches }), {});
    }
    return build_utils_1.glob(globMatcher, entryDir);
}
